import numpy as np

# SINUSOIDAL DISPLACEMENT FIELD

class Sinusoid:
    def __init__(self,w=2*np.pi*np.ones([3]),A=1.):
        self.w = np.array(w)
        self.amplitude = A
        self.offset1 = np.random.uniform(0.,np.pi*2)
        
    def random(self,length=1):
        theta = np.random.uniform(0.,np.pi*2)
        phi = np.random.uniform(-np.pi/2,np.pi/2)
                
        self.w = np.empty([3])
        self.w[0] = length*np.cos(theta)*np.cos(phi)
        self.w[1] = length*np.sin(theta)*np.cos(phi)
        self.w[2] = length*np.sin(phi)
        
        self.offset1 = np.random.uniform(0.,np.pi*2)
        self.offset2 = np.random.uniform(0.,np.pi*2)
        
    def __call__(self,X):
        u1 = self.amplitude*np.cos(np.dot(X,self.w.T)+self.offset1)
        u2 = self.amplitude*np.sin(np.dot(X,self.w.T)+self.offset1)
        u3 = self.amplitude*np.ones(u1.shape)
        return np.vstack([u1,u2,u3]).T
    def grad(self,X):
        grad_u = np.zeros([X.shape[0],3])
        grad_u[0] = -self.amplitude*np.sin(self.w.dot(X)+self.offset1)*self.w
        grad_u[1] = self.amplitude*np.cos(self.w.dot(X)+self.offset1)*self.w
        return grad_u
        
# LINEAR DISPLACEMENT FIELD

class LinearDisplacement:
    def __init__(self,A=1.):
        self.amplitude = A
        self.offset = np.random.normal(scale = 1.,size=[1,3])
        
    def random(self,scale=1):
        self.offset = np.random.normal(scale = scale,size=[1,3])

    def __call__(self,X):
        
        return self.amplitude*X + self.offset
    
    def grad(self,X):
        return self.amplitude*np.eye(3)
